function [beta, nu, lambda, maxl, se_beta, variance_beta] = loglinear_comp(y, xx)
% fits loglinear COMP regression models, as decribed in 
% "Mean-parametrized Conway-Maxwell-Poisson models for dispersed counts" 
% by Alan Huang, University of Queensland

n = length(y) ; % sample size
q = size(xx,2) ; % number of covariates

% starting values for optimization
% beta0 = [log(mean(y)); zeros(q-1,1)]; % start from scratch
% or use fitted Poisson model as starting value
beta0 = glmfit(xx(:,2:q), y, 'poisson') ;
nu0 = 1 ; 
lambda0 = ones(n,1)*mean(y) ;
param0 = [beta0; lambda0; nu0] ;

% pass data to negative loglikelihood function and mean constraints
loglik = @(param)comp_mu_loglik(param, y, xx) ;
conn = @(param)comp_mu_constraints(param, y, xx) ;

% minimize the negative loglikelihood
options = optimset('MaxFunEvals',1e5, 'MaxIter', 1e5, 'TolFun', 1e-8, 'TolCon', 1e-8, 'TolX', 1e-8, 'Algorithm', 'interior-point', 'Display', 'off', 'GradConstr', 'on', 'GradObj', 'on') ;
[param, negmaxl]=fmincon(loglik, param0, [], [], [], [], [], [], conn, options) ;

% post-fitting output
maxl = - negmaxl ; % maximum loglikelihood achieved
beta = param(1:q) ; % estimated regression coefficients beta
lambda = param(q+1:q+n) ; % estimated rates (not generally useful)
nu = param(q+n+1) ; % estimate dispersion

% compute standard errors and variances of estimated betas, if requested
if nargout >  4
    fitted = exp(xx*beta) ;
    precision_beta = 0;
    for i = 1:n
        precision_beta = precision_beta + fitted(i)^2*xx(i,:)'*xx(i,:)/comp_variances(lambda(i), nu) ;
    end
    variance_beta = inv(precision_beta) ;
    se_beta = sqrt(diag(variance_beta)) ;
end
end